package com.sd.business.listener;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.ruoyi.common.constant.StatusConstant;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.mq.config.ArtworkConfig;
import com.sd.mq.entity.ArtworkMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Objects;

/**
 * 图稿监听
 */
@Slf4j
@Component
public class ArtworkListener {

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @RabbitListener(bindings = {
            @QueueBinding(value = @Queue(ArtworkConfig.KX_ARTWORK_QUEUE_NAME), exchange = @Exchange(ArtworkConfig.DIRECT_EXCHANGE_NAME))
    })
    public void msgReceive(ArtworkMessage artworkMessage, Channel channel, Message message) throws IOException {

        log.info("接收更新图稿：{}", JSON.toJSONString(artworkMessage));

        long deliveryTag = message.getMessageProperties().getDeliveryTag();

        try {
            if (Objects.equals(artworkMessage.getType(), StatusConstant.YES)) {
                ArtworkLibrary artworkLibrary = BeanUtil.copyProperties(artworkMessage, ArtworkLibrary.class);
                artworkLibrary.setMesId(artworkLibrary.getId());
                artworkLibrary.setId(null);
                artworkLibrary.setType(1);
                artworkLibrary.setArtworkType(2);
                artworkLibraryService.save(artworkLibrary);
            } else {
                artworkLibraryService.remove(q -> q.eq(ArtworkLibrary::getMesId, artworkMessage.getId()));
            }

            channel.basicAck(deliveryTag, true);
        } catch (Exception e) {
            log.error("接收更新图稿异常", e);
            channel.basicReject(deliveryTag, true);
        }
    }

}
