package com.sd.business.listener;

import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.sku.po.SkuDefaultPackageSpec;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.sku.SkuDefaultPackageSpecService;
import com.sd.mq.config.DefaultPackageSpecConfig;
import com.sd.mq.entity.DefaultPackageSpecMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 默认包材配置规格监听
 */
@Slf4j
@Component
public class DefaultPackageSpecListener {

    @Autowired
    private SkuDefaultPackageSpecService skuDefaultPackageSpecService;

    @Autowired
    private BomSpecService bomSpecService;


    @RabbitListener(bindings = {
            @QueueBinding(value = @Queue(DefaultPackageSpecConfig.DEFAULT_PACKAGE_SPEC_NAME), exchange = @Exchange(DefaultPackageSpecConfig.DIRECT_EXCHANGE_NAME))
    })
    public void msgReceive(DefaultPackageSpecMessage specMessage, Channel channel, Message message) throws IOException {

        log.info("接收修改包材配置：{}", JSON.toJSONString(specMessage));

        long deliveryTag = message.getMessageProperties().getDeliveryTag();

        try {
            if (Objects.equals(specMessage.getType(), StatusConstant.YES)) {
                SkuDefaultPackageSpec defaultPackageSpec = new SkuDefaultPackageSpec();
                defaultPackageSpec.setSpec(specMessage.getSpec());
                // 获取包材所有id
                List<Long> packageBomSpecIds = Stream.of(specMessage.getAccessoryId(),
                                specMessage.getBubblePackId(),
                                specMessage.getColouredPaperId(),
                                specMessage.getLogisticsPackagingMaterialId(),
                                specMessage.getDropId(),
                                specMessage.getMeshBagId(),
                                specMessage.getOppMembraneId(),
                                specMessage.getPeBagId(),
                                specMessage.getPaperBoxId(),
                                specMessage.getOtherPackingMaterialId(),
                                specMessage.getSuspendersId(),
                                specMessage.getSelfAdhesiveStickerId())
                        .filter(Objects::nonNull)
                        .collect(Collectors.toList());

                if (!packageBomSpecIds.isEmpty()) {
                    // 设置包材id
                    Map<Long, Long> bomSpecMap = bomSpecService.mapKV(
                            BomSpec::getMesId,
                            BaseIdPo::getId,
                            q -> q.in(BomSpec::getMesId, packageBomSpecIds));

                    defaultPackageSpec.setAccessoryId(bomSpecMap.get(specMessage.getAccessoryId()));
                    defaultPackageSpec.setBubblePackId(bomSpecMap.get(specMessage.getBubblePackId()));
                    defaultPackageSpec.setColouredPaperId(bomSpecMap.get(specMessage.getColouredPaperId()));
                    defaultPackageSpec.setLogisticsPackagingMaterialId(bomSpecMap.get(specMessage.getLogisticsPackagingMaterialId()));
                    defaultPackageSpec.setDropId(bomSpecMap.get(specMessage.getDropId()));
                    defaultPackageSpec.setMeshBagId(bomSpecMap.get(specMessage.getMeshBagId()));
                    defaultPackageSpec.setOppMembraneId(bomSpecMap.get(specMessage.getOppMembraneId()));
                    defaultPackageSpec.setPeBagId(bomSpecMap.get(specMessage.getPeBagId()));
                    defaultPackageSpec.setPaperBoxId(bomSpecMap.get(specMessage.getPaperBoxId()));
                    defaultPackageSpec.setOtherPackingMaterialId(bomSpecMap.get(specMessage.getOtherPackingMaterialId()));
                    defaultPackageSpec.setSuspendersId(bomSpecMap.get(specMessage.getSuspendersId()));
                    defaultPackageSpec.setSelfAdhesiveStickerId(bomSpecMap.get(specMessage.getSelfAdhesiveStickerId()));
                }

                // 新增配置
                skuDefaultPackageSpecService.save(defaultPackageSpec);
            } else {
                // 删除配置
                skuDefaultPackageSpecService.remove(q -> q.eq(SkuDefaultPackageSpec::getSpec, specMessage.getSpec()));
            }
            channel.basicAck(deliveryTag, true);
        } catch (Exception e) {
            log.error("接收新增bom异常", e);
            channel.basicReject(deliveryTag, true);
        }
    }
}
