package com.sd.business.service.bom.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.dto.BomClassifyDto;
import com.sd.business.entity.bom.dto.BomClassifySelectDto;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.vo.BomClassifyVo;
import com.sd.business.mapper.bom.BomClassifyMapper;
import com.sd.business.service.bom.BomClassifyService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.tree.TreeUtil;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * bom分类 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-20
 */
@Service
public class BomClassifyServiceImpl extends ServiceImpl<BomClassifyMapper, BomClassify> implements BomClassifyService {

    @Override
    public List<BomClassifyVo> tree(BomClassifySelectDto dto) {

        List<BomClassifyVo> list = Sql.create(BomClassifyVo.class)
                .selectAll(BomClassify.class)
                .from(BomClassify.class)

                .like(BomClassify::getCode, dto.getCode())
                .like(BomClassify::getName, dto.getName())
                .orderByAsc(BomClassify::getSort)
                .list();

        return TreeUtil.build(list);
    }

    @Override
    public BomClassifyVo detail(Long id) {

        BomClassifyVo vo = Sql.create(BomClassifyVo.class)
                .selectAll(BomClassify.class)
                .from(BomClassify.class)
                .eq(BomClassify::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(BomClassifyDto dto) {
        save(dto);
    }

    @Override
    public void edit(BomClassifyDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Override
    public List<BomClassifyVo> getList() {
        List<BomClassifyVo> list = Sql.create(BomClassifyVo.class)
                .selectAll(BomClassify.class)
                .from(BomClassify.class)
                .eq(BomClassify::getParentId, 1)
                .orderByAsc(BomClassify::getSort)
                .list();
        for (BomClassifyVo bomClassifyVo : list) {
            String name = bomClassifyVo.getName();
            int index = name.indexOf("裸");
            if (index < 0) {
                continue;
            }
            bomClassifyVo.setName(name.substring(0, index));
        }
        return list;
    }

}
