package com.sd.business.service.dict.impl;

import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.utils.ObsFileUtil;
import com.sd.business.entity.dict.po.DictConfigData;
import com.sd.business.mapper.dict.DictConfigDataMapper;
import com.sd.business.service.dict.DictConfigDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.dict.vo.DictConfigDataVo;
import com.sd.business.entity.dict.dto.DictConfigDataSelectDto;
import com.sd.business.entity.dict.dto.DictConfigDataDto;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.Assert;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 字典配置数据 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-12-05
 */
@Service
public class DictConfigDataServiceImpl extends ServiceImpl<DictConfigDataMapper, DictConfigData> implements DictConfigDataService {

    @Override
    public List<DictConfigDataVo> getList(DictConfigDataSelectDto dto) {

        List<DictConfigDataVo> list = Sql.create(DictConfigDataVo.class)
                .selectAll(DictConfigData.class)
                .from(DictConfigData.class)
                .eq(DictConfigData::getDictCode, dto.getDictCode())
                .like(DictConfigData::getDictValue, dto.getDictValue())
                .orderByDesc(DictConfigData::getId)
                .list();

        return list;
    }

    @Override
    public Page<DictConfigDataVo> getPage(DictConfigDataSelectDto dto) {

        Page<DictConfigDataVo> page = Sql.create(DictConfigDataVo.class)
                .selectAll(DictConfigData.class)
                .from(DictConfigData.class)
                .eq(DictConfigData::getDictCode, dto.getDictCode())
                .like(DictConfigData::getDictValue, dto.getDictValue())
                .orderByDesc(DictConfigData::getId)
                .page(dto);

        return page;
    }

    @Override
    public DictConfigDataVo detail(Long id) {

        DictConfigDataVo vo = Sql.create(DictConfigDataVo.class)
                .selectAll(DictConfigData.class)
                .from(DictConfigData.class)
                .eq(DictConfigData::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(DictConfigDataDto dto) {
        this.save(dto);

        if (dto.getImgId() != null) {
            ObsFile imgFile = new ObsFile();
            imgFile.setId(dto.getImgId());
            ObsFileUtil.saveFile(Collections.singletonList(imgFile), dto.getId(), 1);
        }
    }

    @Override
    public void edit(DictConfigDataDto dto) {
        this.updateById(dto);
        if (dto.getImgId() != null) {
            ObsFile imgFile = new ObsFile();
            imgFile.setId(dto.getImgId());
            ObsFileUtil.editFile(Collections.singletonList(imgFile), dto.getId(), 1);
        }
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
        ObsFileUtil.removeFile(id);
    }

}
