package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.order.dto.BulkOrderConfirmDto;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.enums.ArtworkLogTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuArtworkLog;
import com.sd.business.entity.order.po.OrderSkuArtworkMake;
import com.sd.business.entity.order.vo.BulkOrderVo;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.sku.po.SkuUser;
import com.sd.business.mapper.order.OrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuArtworkLogService;
import com.sd.business.service.order.OrderSkuArtworkMakeService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuUserService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import com.sd.mq.config.OrderConfig;
import com.sd.mq.entity.OrderMessage;
import com.sd.mq.entity.OrderSkuDetail;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-11-23
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderInfo> implements OrderService {

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuArtworkMakeService orderSkuArtworkMakeService;

    @Autowired
    private OrderSkuArtworkLogService orderSkuArtworkLogService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private SkuUserService skuUserService;

    @Override
    public Page<OrderInfoVo> getPage(OrderInfoSelectDto dto) {

        Page<OrderInfoVo> page = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(BomSpec::getCode, OrderInfoVo::getBomSpecCode)
                .selectAs(SkuUser::getCode, OrderInfoVo::getSkuCode)
                .from(OrderInfo.class)
                .innerJoin(OrderSku.class, OrderSku::getOrderId, OrderInfo::getId)
                .leftJoin(BomSpec.class, BomSpec::getId, OrderSku::getBomSpecId)
                .leftJoin(SkuUser.class, SkuUser::getId, OrderSku::getSkuUserId)
                .like(OrderInfo::getCode, dto.getCode())
                .like(SkuUser::getCode, dto.getSkuCode())
                .eq(OrderInfo::getCreateUser, SecurityUtils.getUserId())
                .orderByDesc(OrderInfo::getId)
                .page(dto);

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {

        OrderInfoVo vo = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .from(OrderInfo.class)
                .eq(OrderInfo::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        // 赋值订单明细
        List<OrderSku> tempOrderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuVo> orderSkuList = BeanUtil.copyToList(tempOrderSkuList, OrderSkuVo.class);
        bomSpecService.attributeAssign(orderSkuList, OrderSkuVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });
        skuUserService.attributeAssign(orderSkuList, OrderSkuVo::getSkuUserId, (item, skuUser) -> {
            item.setSkuCode(skuUser.getCode());
            item.setSkuName(skuUser.getName());
        });
        vo.setOrderSkuList(orderSkuList);

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(OrderInfoDto dto) {
        dto.setCode(CodeEnum.ORDER_CODE.getCode());
        this.save(dto);

        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        // 新增订单产品包材配件
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        // 新增订单产品制作图稿数据
        List<OrderSkuArtworkMake> orderSkuArtworkMakeList = tempOrderSkuList.stream()
                .filter(item -> StrUtil.isBlank(item.getProductionDocument()))
                .map(item -> {
                    OrderSkuArtworkMake orderSkuArtworkMake = new OrderSkuArtworkMake();
                    orderSkuArtworkMake.setOrderId(dto.getId());
                    orderSkuArtworkMake.setOrderSkuId(item.getId());
                    orderSkuArtworkMake.setBlueprint(item.getBlueprint());
                    orderSkuArtworkMake.setBlueprintDetails(item.getBlueprintDetails());
                    orderSkuArtworkMake.setStatus(StatusConstant.NO);
                    return orderSkuArtworkMake;
                }).collect(Collectors.toList());
        orderSkuArtworkMakeService.saveBatch(orderSkuArtworkMakeList);

        // 新增订单产品图稿操作日志
        List<OrderSkuArtworkLog> orderSkuArtworkLogList = tempOrderSkuList.stream().map(item -> {
            OrderSkuArtworkLog orderSkuArtworkLog = new OrderSkuArtworkLog();
            orderSkuArtworkLog.setOrderId(dto.getId());
            orderSkuArtworkLog.setOrderSkuId(item.getId());
            orderSkuArtworkLog.setType(ArtworkLogTypeEnum.UPLOAD_BLUEPRINT.getKey());
            orderSkuArtworkLog.setOperator(SecurityUtils.getLoginUser().getUser().getNickName());
            return orderSkuArtworkLog;
        }).collect(Collectors.toList());
        orderSkuArtworkLogService.saveBatch(orderSkuArtworkLogList);

        sendMq(dto);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(OrderInfoDto dto) {
        // 更新订单
        this.updateById(dto);
        // 更新订单产品
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());
    }

    @Override
    public void delete(Long id) {
        removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuArtworkMakeService.remove(q -> q.eq(OrderSkuArtworkMake::getOrderId, id));
        orderSkuArtworkLogService.remove(q -> q.eq(OrderSkuArtworkLog::getOrderId, id));
    }

    @Override
    public Page<BulkOrderVo> getBulkOrderPage(OrderInfoSelectDto dto) {

        Page<BulkOrderVo> page = Sql.create(BulkOrderVo.class)
                .selectAll(OrderInfo.class)
                .select(OrderSku::getLength)
                .select(OrderSku::getWidth)
                .select(OrderSku::getHeight)
                .select(OrderSku::getChromatophore)
                .select(OrderSku::getColour)
                .select(OrderSku::getFrontGrain)
                .select(OrderSku::getReverseGrain)
                .from(OrderInfo.class)
                .innerJoin(OrderSku.class, OrderSku::getOrderId, OrderInfo::getId)
                .like(OrderInfo::getCode, dto.getCode())
                .isNull(OrderSku::getBomSpecId)
                .orderByDesc(OrderInfo::getId)
                .page(dto);

        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void bulkOrderConfirm(BulkOrderConfirmDto dto) {
        OrderInfo orderInfo = this.getById(dto.getId());
        Assert.notNull(orderInfo, "未知数据");
        if (Objects.equals(dto.getIsConfirm(), StatusConstant.YES)) {
            Assert.notNull(dto.getBomSpecId(), "请选择bom");
            BomSpec bomSpec = bomSpecService.getById(dto.getBomSpecId());
            Assert.notNull(bomSpec, "未知bom");
            OrderSku orderSku = orderSkuService.getOne(q -> q.eq(OrderSku::getOrderId, dto.getId()));
            orderSku.setBomSpecId(bomSpec.getId());
            orderSkuService.updateById(orderSku);
            orderInfo.setStatus(OrderStatusEnum.PENDING_PAYMENT.getKey());
        } else {
            orderInfo.setStatus(OrderStatusEnum.REJECT.getKey());
        }
        this.updateById(orderInfo);
    }

    /**
     * 发送消息到mq
     * @param dto
     */
    private void sendMq(OrderInfoDto dto) {
        if (!Objects.equals(dto.getStatus(), OrderStatusEnum.WAIT_FOR_DELIVERY.getKey())) {
            return;
        }
        OrderMessage orderMessage = new OrderMessage();
        BeanUtil.copyProperties(dto, orderMessage);
        orderMessage.setOperator(SecurityUtils.getLoginUser().getUser().getNickName());

        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        List<Long> bomSpecIds = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, Long> bomSpecMap = bomSpecService.mapKV(BaseIdPo::getId, BomSpec::getMesId, q -> q.in(BaseIdPo::getId, bomSpecIds));

        List<OrderSkuDetail> orderSkuDetailList = orderSkuList.stream().map(item -> {
            OrderSkuDetail orderSkuDetail = new OrderSkuDetail();
            orderSkuDetail.setBomSpecId(bomSpecMap.get(item.getBomSpecId()));
            orderSkuDetail.setPackagingSet(item.getPackagingSet());
            orderSkuDetail.setQuantity(item.getQuantity());
            orderSkuDetail.setPrintType(item.getPrintType());
            orderSkuDetail.setBlueprint(item.getProofingImg());
            orderSkuDetail.setProductionDocument(item.getProductionDocument());
            return orderSkuDetail;
        }).collect(Collectors.toList());
        orderMessage.setOrderSkuDetailList(orderSkuDetailList);
        RabbitMqUtil.send(OrderConfig.DIRECT_EXCHANGE_NAME, OrderConfig.KX_ORDER_ADD_NAME, orderMessage);
    }
}
