package com.sd.business.service.order.impl;

import cn.hutool.core.util.NumberUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.order.dto.OrderSkuArtworkLogDto;
import com.sd.business.entity.order.enums.ArtworkLogTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuArtworkMake;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.sku.po.SkuUser;
import com.sd.business.mapper.order.OrderSkuMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuArtworkLogService;
import com.sd.business.service.order.OrderSkuArtworkMakeService;
import com.sd.business.service.order.OrderSkuService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 订单sku 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-11-23
 */
@Service
public class OrderSkuServiceImpl extends ServiceImpl<OrderSkuMapper, OrderSku> implements OrderSkuService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuArtworkLogService orderSkuArtworkLogService;

    @Autowired
    private OrderSkuArtworkMakeService orderSkuArtworkMakeService;

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public List<OrderSkuVo> getUnconfirmedArtworkList(Long orderId) {

        List<OrderSkuVo> list = Sql.create(OrderSkuVo.class)
                .selectAll(OrderSku.class)
                .selectAs(SkuUser::getCode, OrderSkuVo::getSkuCode)
                .selectAs(SkuUser::getName, OrderSkuVo::getSkuName)
                .selectAs(BomSpec::getCode, OrderSkuVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuVo::getBomSpecName)
                .from(OrderSku.class)
                .innerJoin(OrderSkuArtworkMake.class, OrderSkuArtworkMake::getOrderSkuId, OrderSku::getId)
                .innerJoin(BomSpec.class, BomSpec::getId, OrderSku::getBomSpecId)
                .leftJoin(SkuUser.class, SkuUser::getId, OrderSku::getSkuUserId)
                .eq(OrderSku::getOrderId, orderId)
                .eq(OrderSkuArtworkMake::getStatus, 1)
                .orderByDesc(OrderSku::getId)
                .list();

        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void confirmArtwork(OrderSkuArtworkLogDto dto) {
        OrderSku orderSku = this.getById(dto.getOrderSkuId());
        Assert.notNull(orderSku, "未知数据");
        dto.setOperator(SecurityUtils.getLoginUser().getUser().getNickName());
        orderSkuArtworkLogService.save(dto);
        OrderSkuArtworkMake orderSkuArtworkMake = orderSkuArtworkMakeService.getOne(q -> q.eq(OrderSkuArtworkMake::getOrderSkuId, orderSku.getId()));

        if (Objects.equals(dto.getType(), ArtworkLogTypeEnum.CONFIRM_PROOFING_IMG.getKey())) {
            orderSkuArtworkMake.setStatus(3);
            orderSku.setProofingImg(orderSkuArtworkMake.getProofingImg());
            orderSku.setProductionDocument(orderSkuArtworkMake.getProductionDocument());
            // 查询订单sku是否全部确认图稿，全部通过修改订单状态
            long count = orderSkuArtworkMakeService.count(q -> q.eq(OrderSkuArtworkMake::getOrderId, orderSku.getOrderId())
                    .ne(OrderSkuArtworkMake::getStatus, 3)
                    .ne(BaseIdPo::getId, orderSkuArtworkMake.getId()));
            if (count == 0) {
                orderService.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.WAIT_FOR_DELIVERY.getKey())
                        .eq(BaseIdPo::getId, orderSku.getOrderId()));
            }
            // 新增专属图稿
            BomSpecBo bomSpecBo = bomSpecService.getBomSpecBoById(orderSku.getBomSpecId());
            String classify = "";
            String classifyName = bomSpecBo.getClassifyName();
            int nameIndex = classifyName.indexOf("裸");
            if (nameIndex > 0) {
                classify = classifyName.substring(0, nameIndex) + "_";
            }
            String artworkName = classify + orderSku.getLength().stripTrailingZeros().toPlainString() + "_" + orderSku.getWidth().stripTrailingZeros().toPlainString();
            ArtworkLibrary library = artworkLibraryService.getOne(q -> q
                    .likeRight(ArtworkLibrary::getArtworkName, artworkName)
                    .orderByDesc(ArtworkLibrary::getCreateTime));
            int num = 1;
            if (library != null) {
                String name = library.getArtworkName();
                String index = name.substring(library.getArtworkName().lastIndexOf("_") + 1);
                if (NumberUtil.isInteger(index)) {
                    num = Integer.parseInt(index) + 1;
                }
            }
            ArtworkLibrary artworkLibrary = new ArtworkLibrary();
            artworkLibrary.setType(2);
            artworkLibrary.setArtworkType(2);
            artworkLibrary.setLength(orderSku.getLength());
            artworkLibrary.setWidth(orderSku.getWidth());
            artworkLibrary.setArtworkName(artworkName + "_" + num);
            artworkLibrary.setImgUrl(orderSku.getProofingImg());
            artworkLibrary.setImgName(artworkLibrary.getArtworkName() +
                    artworkLibrary.getImgUrl().substring(artworkLibrary.getImgUrl().lastIndexOf(".")));
            artworkLibrary.setFileUrl(orderSku.getProductionDocument());
            artworkLibrary.setFileName(artworkLibrary.getArtworkName() +
                    artworkLibrary.getFileUrl().substring(artworkLibrary.getFileUrl().lastIndexOf(".")));
            artworkLibraryService.save(artworkLibrary);
            this.updateById(orderSku);
        } else {
            orderSkuArtworkMake.setStatus(2);
        }

        orderSkuArtworkMakeService.updateById(orderSkuArtworkMake);
    }
}
